#ifndef PGTRANSACTOR_H
#define PGTRANSACTOR_H

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <iostream>
#include <string>

#ifdef HAVE_PQXX
#include <pqxx/connection>
#include <pqxx/except>
#include <pqxx/transaction>
#endif

template<typename Transactor>
class PGTransactor {
public:
	typedef pqxx::read_transaction transaction_t;

	PGTransactor(pqxx::connection& conn, const std::string& tname = "transactor")
		: m_tran(conn, tname)
	{
	}

	void operator()(void) {
		try {
			transactor().run();
			m_tran.commit();
		} catch (const pqxx::in_doubt_error& e) {
			transactor().on_doubt();
			throw;
		} catch (const std::exception& e) {
			transactor().on_abort(e.what());
			throw;
		} catch (...) {
			transactor().on_abort("Unknown exception");
			throw;
		}
		transactor().on_commit();
	}

protected:
	transaction_t m_tran;

	Transactor& transactor(void) {
		return *static_cast<Transactor*>(this);
	}

	void run(void) {}
	void on_abort(const char msg[]) noexcept {
		std::cerr << "pg: " << m_tran.name() << ": transaction aborted: " << msg << std::endl;
	}
	void on_doubt(void) noexcept {
		std::cerr << "pg: " << m_tran.name() << ": transaction in doubt" << std::endl;
	}
	void on_commit(void) {}
};

#endif /* PGTRANSACTOR_H */
